"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndyVdrIndyDidResolver = void 0;
const indy_vdr_shared_1 = require("@hyperledger/indy-vdr-shared");
const error_1 = require("../error");
const pool_1 = require("../pool");
const didIndyUtil_1 = require("./didIndyUtil");
const didSovUtil_1 = require("./didSovUtil");
class IndyVdrIndyDidResolver {
    constructor() {
        this.supportedMethods = ['indy'];
    }
    async resolve(agentContext, did) {
        const didDocumentMetadata = {};
        try {
            const nym = await this.getPublicDid(agentContext, did);
            // Get DID Document from Get NYM response
            const didDocument = await this.buildDidDocument(agentContext, nym, did);
            return {
                didDocument,
                didDocumentMetadata,
                didResolutionMetadata: { contentType: 'application/did+ld+json' },
            };
        }
        catch (error) {
            return {
                didDocument: null,
                didDocumentMetadata,
                didResolutionMetadata: {
                    error: 'notFound',
                    message: `resolver_error: Unable to resolve did '${did}': ${error}`,
                },
            };
        }
    }
    async buildDidDocument(agentContext, getNymResponseData, did) {
        // Create base Did Document
        // For modern did:indy DIDs, we assume that GET_NYM is always a full verkey in base58.
        // For backwards compatibility, we accept a shortened verkey and convert it using previous convention
        const verkey = (0, didSovUtil_1.getFullVerkey)(did, getNymResponseData.verkey);
        const builder = (0, didIndyUtil_1.indyDidDocumentFromDid)(did, verkey);
        // If GET_NYM does not return any diddocContent, fallback to legacy GET_ATTRIB endpoint
        if (!getNymResponseData.diddocContent) {
            const keyAgreementId = `${did}#key-agreement-1`;
            const endpoints = await this.getEndpointsForDid(agentContext, did);
            if (endpoints) {
                // If there is at least a didcomm endpoint, generate and a key agreement key
                const commTypes = ['endpoint', 'did-communication', 'DIDComm'];
                if (commTypes.some((type) => { var _a; return (_a = endpoints.types) === null || _a === void 0 ? void 0 : _a.includes(type); })) {
                    builder
                        .addVerificationMethod({
                        controller: did,
                        id: keyAgreementId,
                        publicKeyBase58: (0, didIndyUtil_1.createKeyAgreementKey)(getNymResponseData.verkey),
                        type: 'X25519KeyAgreementKey2019',
                    })
                        .addKeyAgreement(keyAgreementId);
                }
                // Process endpoint attrib following the same rules as for did:sov
                (0, didSovUtil_1.addServicesFromEndpointsAttrib)(builder, did, endpoints, keyAgreementId);
            }
            return builder.build();
        }
        else {
            // Combine it with didDoc (TODO: Check if diddocContent is returned as a JSON object or a string)
            return (0, didIndyUtil_1.combineDidDocumentWithJson)(builder.build(), getNymResponseData.diddocContent);
        }
    }
    async getPublicDid(agentContext, did) {
        const indyVdrPoolService = agentContext.dependencyManager.resolve(pool_1.IndyVdrPoolService);
        const { namespace, id } = (0, didIndyUtil_1.parseIndyDid)(did);
        const pool = indyVdrPoolService.getPoolForNamespace(namespace);
        const request = new indy_vdr_shared_1.GetNymRequest({ dest: id });
        const didResponse = await pool.submitReadRequest(request);
        if (!didResponse.result.data) {
            throw new error_1.IndyVdrNotFoundError(`DID ${id} not found in indy namespace ${namespace}`);
        }
        return JSON.parse(didResponse.result.data);
    }
    async getEndpointsForDid(agentContext, did) {
        var _a;
        const indyVdrPoolService = agentContext.dependencyManager.resolve(pool_1.IndyVdrPoolService);
        const { namespace, id } = (0, didIndyUtil_1.parseIndyDid)(did);
        const pool = indyVdrPoolService.getPoolForNamespace(namespace);
        try {
            agentContext.config.logger.debug(`Get endpoints for did '${id}' from ledger '${pool.indyNamespace}'`);
            const request = new indy_vdr_shared_1.GetAttribRequest({ targetDid: id, raw: 'endpoint' });
            agentContext.config.logger.debug(`Submitting get endpoint ATTRIB request for did '${id}' to ledger '${pool.indyNamespace}'`);
            const response = await pool.submitReadRequest(request);
            if (!response.result.data) {
                return null;
            }
            const endpoints = (_a = JSON.parse(response.result.data)) === null || _a === void 0 ? void 0 : _a.endpoint;
            agentContext.config.logger.debug(`Got endpoints '${JSON.stringify(endpoints)}' for did '${did}' from ledger '${pool.indyNamespace}'`, {
                response,
                endpoints,
            });
            return endpoints;
        }
        catch (error) {
            agentContext.config.logger.error(`Error retrieving endpoints for did '${did}' from ledger '${pool.indyNamespace}'`, {
                error,
            });
            throw new error_1.IndyVdrError(error);
        }
    }
}
exports.IndyVdrIndyDidResolver = IndyVdrIndyDidResolver;
//# sourceMappingURL=IndyVdrIndyDidResolver.js.map